#!/usr/bin/env python

"""
receiver.py - Demonstrates how to use XQuery from Python.

Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
All rights reserved.
Contact: Nokia Corporation (qt-info@nokia.com)

This file is part of the documentation of Qt. It was originally
published as part of Qt Quarterly.

$QT_BEGIN_LICENSE:LGPL$
Commercial Usage
Licensees holding valid Qt Commercial licenses may use this file in
accordance with the Qt Commercial License Agreement provided with the
Software or, alternatively, in accordance with the terms contained in
a written agreement between you and Nokia.

GNU Lesser General Public License Usage
Alternatively, this file may be used under the terms of the GNU Lesser
General Public License version 2.1 as published by the Free Software
Foundation and appearing in the file LICENSE.LGPL included in the
packaging of this file.  Please review the following information to
ensure the GNU Lesser General Public License version 2.1 requirements
will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.

In addition, as a special exception, Nokia gives you certain additional
rights.  These rights are described in the Nokia Qt LGPL Exception
version 1.1, included in the file LGPL_EXCEPTION.txt in this package.

GNU General Public License Usage
Alternatively, this file may be used under the terms of the GNU
General Public License version 3.0 as published by the Free Software
Foundation and appearing in the file LICENSE.GPL included in the
packaging of this file.  Please review the following information to
ensure the GNU General Public License version 3.0 requirements will be
met: http://www.gnu.org/copyleft/gpl.html.

If you have questions regarding the use of this file, please contact
Nokia at qt-info@nokia.com.
$QT_END_LICENSE$
"""

import sys
from PyQt4.QtCore import QCoreApplication, QStringList, QVariant
from PyQt4.QtXmlPatterns import QAbstractXmlReceiver, QXmlItem, QXmlQuery

query_string = """declare variable $url external;

doc($url)//sun,
for $time in doc($url)//tabular/time
    order by $time/@from
    return <time>
        { $time/@from }
        <temperature>
            { $time/temperature/@value, $time/temperature/@unit }
        </temperature>
        </time>
"""

class Receiver(QAbstractXmlReceiver):

    def __init__(self, namePool):
    
        QAbstractXmlReceiver.__init__(self)
        self.namePool = namePool
        self.sun = {}
        self.forecasts = []
        self._stack = []
        self._current_forecast = {}
    
    def attribute(self, xmlname, value):
    
        name = unicode(xmlname.localName(self.namePool))
        value = unicode(value.toString())
        current_element = self._stack[-1]
        
        if current_element == "time":
            self._current_forecast[name] = value
        
        elif current_element == "temperature":
            d = self._current_forecast.setdefault("temperature", {})
            d[name] = value
    
    def characters(self, value):
        pass
    
    def comment(self, value):
        pass
    
    def namespaceBinding(self, xmlname):
        pass
    
    def startElement(self, xmlname):
        name = xmlname.localName(self.namePool)
        self._stack.append(unicode(name))
        
        if name == "time":
            self._current_forecast = {}
    
    def endElement(self):
        if self._stack.pop() == "time":
            self.forecasts.append(self._current_forecast)
    
    def startOfSequence(self):
        pass
    
    def endOfSequence(self):
        pass


if __name__ == "__main__":

    app = QCoreApplication(sys.argv)
    
    if len(app.arguments()) != 2:
    
        sys.stderr.write("Usage: %s <URL of XML file>\n" % app.arguments()[0])
        sys.exit(1)
    
    url = app.arguments()[1]
    
    query = QXmlQuery()
    query.bindVariable("url", QXmlItem(QVariant(url)))
    query.setQuery(query_string)
    receiver = Receiver(query.namePool())
    
    if query.isValid():
        if query.evaluateTo(receiver):
            for forecast in receiver.forecasts:
                print forecast["from"]
                print forecast["temperature"]["value"], forecast["temperature"]["unit"]
    
    sys.exit(0)
